@@ -6,6 +6,8 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
+use std::collections::BTreeSet;
+
 use super::{
     messaging::{send, send_to_nodes},
     role::{AdultRole, Role},
@@ -63,6 +65,14 @@ impl Node {
                     Ok(vec![self.push_state(our_prefix, msg_id).await?])
                 }
             }
+            NodeDuty::AdultsChanged(adults) => {
+                let our_name = self.our_name().await;
+                Ok(self
+                    .role
+                    .as_adult_mut()?
+                    .handle_adults_changed(adults, our_name)
+                    .await)
+            }
             NodeDuty::SectionSplit {
                 our_key,
                 our_prefix,
@@ -170,8 +180,6 @@ impl Node {
                 let elder = self.role.as_elder_mut()?;
                 elder.section_funds.remove_node_wallet(name);
 
-                ops.extend(elder.meta_data.remove_and_replicate_chunks(name).await?);
-
                 Ok(ops)
             }
             //
@@ -189,6 +197,7 @@ impl Node {
                 let capacity = self.used_space.max_capacity().await;
                 self.role = Role::Adult(AdultRole {
                     chunks: Chunks::new(self.node_info.root_dir.as_path(), capacity).await?,
+                    adult_list: BTreeSet::new(),
                 });
                 Ok(vec![])
             }
@@ -254,11 +263,7 @@ impl Node {
             }
             //
             // -------- Immutable chunks --------
-            NodeDuty::ReadChunk {
-                read,
-                msg_id,
-                origin,
-            } => {
+            NodeDuty::ReadChunk { read, msg_id, .. } => {
                 // TODO: remove this conditional branching
                 // routing should take care of this
                 let data_section_addr = read.dst_address();
@@ -269,23 +274,12 @@ impl Node {
                     .matches(&&data_section_addr)
                 {
                     let adult = self.role.as_adult_mut()?;
-                    let mut ops = adult.chunks.read(&read, msg_id).await?;
+                    let mut ops = adult.chunks.read(&read, msg_id);
                     ops.extend(adult.chunks.check_storage().await?);
                     Ok(ops)
                 } else {
-                    Ok(vec![NodeDuty::Send(OutgoingMsg {
-                        msg: Message::NodeQuery {
-                            query: NodeQuery::Chunks {
-                                query: read,
-                                origin,
-                            },
-                            id: msg_id,
-                        },
-                        dst: DstLocation::Section(data_section_addr),
-                        // TBD
-                        section_source: false,
-                        aggregation: Aggregation::None,
-                    })])
+                    log::error!("MessageId: {:?} NodeCmd::Chunks {:?} should not have reached a wrong Adult", msg_id, read);
+                    Ok(vec![])
                 }
             }
             NodeDuty::WriteChunk {
@@ -296,6 +290,11 @@ impl Node {
                 let adult = self.role.as_adult_mut()?;
                 Ok(vec![adult.chunks.write(&write, msg_id, origin).await?])
             }
+            NodeDuty::ProcessRepublish { chunk, msg_id } => {
+                log::info!("Processing republish with MessageId: {:?}", msg_id);
+                let elder = self.role.as_elder_mut()?;
+                Ok(vec![elder.meta_data.republish_chunk(chunk).await?])
+            }
             NodeDuty::ReachingMaxCapacity => Ok(vec![self.notify_section_of_our_storage().await?]),
             //
             // ------- Misc ------------
@@ -359,12 +358,10 @@ impl Node {
                 src,
             } => {
                 let elder = self.role.as_elder_mut()?;
-                Ok(vec![
-                    elder
-                        .meta_data
-                        .record_adult_write_liveness(correlation_id, result, src)
-                        .await?,
-                ])
+                Ok(elder
+                    .meta_data
+                    .record_adult_write_liveness(correlation_id, result, src)
+                    .await)
             }
             NodeDuty::RecordAdultReadLiveness {
                 response,
@@ -388,27 +385,9 @@ impl Node {
                 elder.section_funds.add_payment(credit);
                 Ok(vec![])
             }
-            NodeDuty::ReplicateChunk(data) => {
+            NodeDuty::ReplicateChunk { data, id } => {
                 let adult = self.role.as_adult_mut()?;
-                adult.chunks.store_for_replication(data).await?;
-                Ok(vec![])
-            }
-            NodeDuty::ReturnChunkToElders {
-                address,
-                id,
-                section,
-            } => {
-                let adult = self.role.as_adult()?;
-                Ok(vec![
-                    adult
-                        .chunks
-                        .get_chunk_for_replication(address, id, section)
-                        .await?,
-                ])
-            }
-            NodeDuty::FinishReplication(data) => {
-                let elder = self.role.as_elder_mut()?;
-                Ok(vec![elder.meta_data.finish_chunk_replication(data).await?])
+                Ok(vec![adult.chunks.store_for_replication(data, id).await?])
             }
             NodeDuty::NoOp => Ok(vec![]),
         }
